from time import sleep

# Import the preset TestFixture.
from .Ethernet_check_fixture import FixtureEthernetCheck, EthProtocolType

from xtr import logging, Severity
logger = logging.getLogger(__name__)


# Create the test case class, that inherits from the preset TestFixture.
class TestEncodeDecodeSomeipMessage(FixtureEthernetCheck):

    # This method will be executed just before
    # the test case method execution.
    def setUp(self):
        logger.info("Setup test case")

    # This method will be executed just after
    # the test case method execution.
    def tearDown(self):
        logger.info("Tear down test case")

    # Create the test case core method that holds the main test steps.
    def test_encode_decode_someip_message(self):

        expected_someip_header_dict = {
            'service_identifier': '0x101',
            'method_identifier': '0x8005',
            'length': 13,
            'client_id': 0,
            'session_id': 0,
            'message_type': 'NOTIFICATION',
            'return_code': 'E_OK',
            'protocol_version': 1,
            'interface_version': 1,
            'message_id': '0x1018005',
            'request_id': 0}

        FixtureEthernetCheck.bus_manager.ethernet_listener(bus_name="Ethernet_MTF",
                                                           protocol=EthProtocolType.IS_SOME_IP).start_listening()

        msg = self.bus_manager.create_ethernet_message(EthProtocolType.IS_SOME_IP)

        someip_msg_payload = "00e04c1356ed001cd77c2fbf080045000031016e00004011aa0ca030c763a030c77d772a771b001dbd17010180050000000d00000000010102000100000000"
        msg.set_all_bytes(bytearray.fromhex(someip_msg_payload))
        msg.someip_header.service_identifier=0x101
        msg.someip_header.method_identifier=0x8005
        msg.someip_header.length=13
        msg.someip_header.client_id=0
        msg.someip_header.session_id=0
        msg.someip_header.protocol_version=1
        msg.someip_header.interface_version=1
        msg.someip_header.message_id=0x1018005
        msg.someip_header.request_id=0

        self.someip_transmitter.send_someip_packet_over_udp("Ethernet_MTF",
                                                            msg)
        sleep(2)
        self.bus_manager.ethernet_listener(bus_name="Ethernet_MTF",
                                           protocol=EthProtocolType.IS_SOME_IP).stop_listening()

        messages = self.bus_manager.ethernet_listener(bus_name="Ethernet_MTF",
                                                      protocol=EthProtocolType.IS_SOME_IP).get_queue()

        for someip_message in messages:
            actual_someip_header_dict = {}
            actual_someip_header_dict['service_identifier'] = hex(
                someip_message['someip_message'].someip_header.service_identifier)
            actual_someip_header_dict['method_identifier'] = hex(
                someip_message['someip_message'].someip_header.method_identifier)
            actual_someip_header_dict['length'] = someip_message['someip_message'].someip_header.length
            actual_someip_header_dict['client_id'] = someip_message['someip_message'].someip_header.client_id
            actual_someip_header_dict['session_id'] = someip_message['someip_message'].someip_header.session_id
            actual_someip_header_dict['message_type'] = someip_message['someip_message'].someip_header.message_type.ToString(
            )
            actual_someip_header_dict['return_code'] = someip_message['someip_message'].someip_header.return_code.ToString(
            )
            actual_someip_header_dict['protocol_version'] = someip_message['someip_message'].someip_header.protocol_version
            actual_someip_header_dict['interface_version'] = someip_message['someip_message'].someip_header.interface_version
            actual_someip_header_dict['message_id'] = hex(
                someip_message['someip_message'].someip_header.message_id)
            actual_someip_header_dict['request_id'] = someip_message['someip_message'].someip_header.request_id

        self.assertEqual(actual_someip_header_dict,
                         expected_someip_header_dict,
                         Severity.BLOCKER)
